# Copyright (C) 2001-2011, Christof Meerwald
# http://jabrss.cmeerw.org, http://www.jotwewe.de/de/xmpp/jabrss

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

class FlexibleLocker:
    def __init__(self, lock, active = True):
        self._lock, self._active = lock, active
        self._locked = False

    def __enter__(self):
        self.lock()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.unlock()

    def locked(self):
        if not self._locked and self._active and self._lock != None:
            self._locked = True

    def lock(self):
        if not self._locked and self._active and self._lock != None:
            self._lock.acquire()
            self._locked = True

    def unlock(self):
        if self._locked and self._active and self._lock != None:
            self._lock.release()
            self._locked = False

    def replace(self, lock):
        self.unlock()
        self._lock = lock
        self.lock()
